local FOLDER = "__plc__"
local img_blank = { filename = FOLDER.."/graphics/blank.png", priority = "extra-high", width = 1, height = 1, frame_count = 1, shift = {0, 0}, }
local img_led_output = { filename = "__base__/graphics/entity/combinator/activity-leds/constant-combinator-LED-N.png", width = 11, height = 10, frame_count = 1, shift = {-0.32, 0.20625}, }
local img_led_input = { filename = "__base__/graphics/entity/combinator/activity-leds/constant-combinator-LED-N.png", width = 11, height = 10, frame_count = 1, shift = {0.296875, -0.40625}, }
local con_point_output = { shadow = { red = {0.15, 	-0.105}, 	green = {0.15, 	0.11}, }, 
							wire  = { red = {0.17, 	-0.045}, 	green = {0.17,	0.17}, } }
local con_point_input = {  shadow = { red = {-0.18, -0.105}, 	green = {-0.18, 0.11}, }, 
							wire  = { red = {-0.20, -0.045}, 	green = {-0.20, 0.17}, } }

data:extend({
    {
        type = "item",
        name = "plc-unit",
        icon = FOLDER.."/graphics/icon/plc.png",
        icon_size = 128,
        flags = {"goes-to-quickbar"},
        subgroup = "plc",
        order = "b",
        stack_size = 50,
        place_result = "plc-unit",
    },
    {
        type = "item",
        name = "plc-input",
        icon = FOLDER.."/graphics/icon/plc.png",
        icon_size = 128,
        flags = {"goes-to-quickbar", "hidden"},
        subgroup = "plc",
        order = "z[plc]",
        stack_size = 50,
        place_result = "plc-input",
    },
    {
        type = "item",
        name = "plc-output",
        icon = FOLDER.."/graphics/icon/plc.png",
        icon_size = 128,
        flags = {"goes-to-quickbar", "hidden"},
        subgroup = "plc",
        order = "z[plc]",
        stack_size = 50,
        place_result = "plc-output",
    },
    {
        type = "item",
        name = "plc-power",
        icon = FOLDER.."/graphics/icon/plc.png",
        icon_size = 128,
        flags = {"goes-to-quickbar", "hidden"},
        subgroup = "plc",
        order = "z[plc]",
        stack_size = 50,
        place_result = "plc-power",
    },
	{
        type = "constant-combinator",
        name = "plc-output",
        icon = FOLDER.."/graphics/icon/plc.png",
        icon_size = 128,
        flags = {"placeable-player", "player-creation", "placeable-off-grid", "not-deconstructable", "not-repairable"},
        mineable = nil,
        fast_replaceable_group = "plc-output",
        order = "y",
        max_health = 10000,
        healing_per_tick = 10000,
        corpse = "small-remnants",
        collision_box = {{-0.0, -0.0}, {0.0, 0.0}},
        collision_mask = {"not-colliding-with-itself"},
        selection_box = {{-0.4, -0.4}, {0.4, 0.4}},
        item_slot_count = 1000,
        sprites = { north = img_blank, east = img_blank, south = img_blank, west = img_blank },
        activity_led_sprites = { north = img_led_output, east = img_led_output, south = img_led_output, west = img_led_output },
        activity_led_light = { intensity = 0.8, size = 1, },
        activity_led_light_offsets = { {0, 0}, {0, 0}, {0, 0}, {0, 0} },
        circuit_wire_connection_points = { con_point_output, con_point_output, con_point_output, con_point_output},
        circuit_wire_max_distance = 10
    }, 
	{
        type = "constant-combinator",
        name = "plc-input",
        icon = FOLDER.."/graphics/icon/plc.png",
        icon_size = 128,
        flags = {"placeable-player", "player-creation", "placeable-off-grid", "not-deconstructable", "not-repairable"},
        mineable = nil,
        fast_replaceable_group = "plc-input",
        order = "y",
        max_health = 10000,
        healing_per_tick = 10000,
        corpse = "small-remnants",
        collision_box = {{-0.0, -0.0}, {0.0, 0.0}},
        collision_mask = {"not-colliding-with-itself"},
        selection_box = {{-0.4, -0.4}, {0.4, 0.4}},
        item_slot_count = 0,
        sprites = { north = img_blank, east = img_blank, south = img_blank, west = img_blank },
        activity_led_sprites = { north = img_led_input, east = img_led_input, south = img_led_input, west = img_led_input },
        activity_led_light = { intensity = 0.8, size = 1, },
        activity_led_light_offsets = { {0, 0}, {0, 0}, {0, 0}, {0, 0} },
        circuit_wire_connection_points = { con_point_input, con_point_input, con_point_input, con_point_input },
        circuit_wire_max_distance = 10
    }, 
	{
        type = "storage-tank",
		name = "plc-unit",
        icon = FOLDER.."/graphics/icon/plc.png",
        icon_size = 128,
        flags = {"placeable-player", "player-creation"},
        minable = {hardness = 0.2, mining_time = 2, result = "plc-unit"},
		max_health = 150,
        corpse = "big-remnants",
        resistances = { { type = "fire", percent = 70 } },
        collision_box = {{-1.4, -1.4}, {1.4, 1.4}},
        selection_box = {{-0.9, -1.4}, {0.9, 1.4}},
        energy_source = { type = "electric", usage_priority = "secondary-input" },
        energy_usage = "250kW",
        vehicle_impact_sound = { filename = "__base__/sound/car-metal-impact.ogg", volume = 0.65 },
        --working_sound = { sound = { filename = "__base__/sound/radar.ogg", volume = 0.3 }, max_sounds_per_type = 3, audible_distance_modifier = 0.5, probability = 1 / (10 * 60) },
        --pictures = { filename = FOLDER.."/graphics/entity/plcmain-grid.png", priority = "low", width = 120, height = 128, apply_projection = false, line_length = 8, direction_count = 64, shift = {0.1875, 0}, animation_speed = 0.5 },
    	pictures = { 
			picture = { sheet = { filename = FOLDER.."/graphics/entity/plc.png", frames = 1, width = 250, height = 200, shift = {0.34375, 0.0}, },},
			fluid_background = { filename = FOLDER.."/graphics/blank.png", priority = "high", width = 1, height = 1, },
			window_background = { filename = FOLDER.."/graphics/blank.png", priority = "high", width = 1, height = 1, },
			flow_sprite = { filename = FOLDER.."/graphics/blank.png", priority = "high", width = 1, height = 1, },
			gas_flow = { filename = FOLDER.."/graphics/blank.png", priority = "high", width = 1, height = 1, frame_count = 1, },
		},
		window_bounding_box = {{0,0},{0,0}},
		fluid_box = { base_area = 1, pipe_covers = pipecoverspictures(), pipe_connections = {},	},
		flow_length_in_ticks = 1,
		circuit_wire_connection_points = circuit_connector_definitions["storage-tank"].points,
		circuit_connector_sprites = circuit_connector_definitions["storage-tank"].sprites,
		circuit_wire_max_distance = 0,
		map_color = {r = 0.8, g = 0.7, b = 0.55},
	},
	{
		type = "assembling-machine",
		name = "plc-power",
        icon_size = 128,
		icon = FOLDER.."/graphics/icon/plc.png",
		flags = {"placeable-player", "player-creation", "placeable-off-grid", "not-deconstructable", "not-repairable"},
        minable = nil,
		max_health = 150,
		corpse = "small-remnants",
		collision_box = {{-0.9, -0.9}, {0.9, 0.9}},
		selection_box = {{-0.0, -0.0}, {0.0, 0.0}},
		collision_mask = {"not-colliding-with-itself"},
        selectable_in_game = false,
        fluid_boxes = {},
		animation = { filename = FOLDER.."/graphics/blank.png", priority = "low", width = 1, height = 1, frame_count = 1, line_length = 1, shift = {0.0, 0.0} },
		vehicle_impact_sound =  {filename = "__base__/sound/car-metal-impact.ogg", volume = 0.65},
		open_sound = {filename = "__base__/sound/machine-open.ogg", volume = 0.85},
		close_sound = {filename = "__base__/sound/machine-close.ogg", volume = 0.75},
		working_sound = { sound = { filename = "__base__/sound/accumulator-working.ogg", volume = 0.0 }, idle_sound = { filename = "__base__/sound/accumulator-idle.ogg", volume = 0.0 }, apparent_volume = 1.5, },
		crafting_categories = {"plc-power"},
		fixed_recipe = "plc-power",
		crafting_speed = 1.0,
		energy_source = { type = "electric", usage_priority = "secondary-input", emissions = 0 },
		energy_usage = "120kW",
		ingredient_count = 0,
		module_specification = {},
		allowed_effects = {}
	},

})

